/*
* Copyright (C) 2022, Ing.-Buero Dr. Michael Lehning, Hildesheim
* Copyright (C) 2022, SICK AG, Waldkirch
* All rights reserved.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*       http://www.apache.org/licenses/LICENSE-2.0
*
*   Unless required by applicable law or agreed to in writing, software
*   distributed under the License is distributed on an "AS IS" BASIS,
*   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*   See the License for the specific language governing permissions and
*   limitations under the License.
*
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of Osnabrueck University nor the names of its
*       contributors may be used to endorse or promote products derived from
*       this software without specific prior written permission.
*     * Neither the name of SICK AG nor the names of its
*       contributors may be used to endorse or promote products derived from
*       this software without specific prior written permission
*     * Neither the name of Ing.-Buero Dr. Michael Lehning nor the names of its
*       contributors may be used to endorse or promote products derived from
*       this software without specific prior written permission
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*
*      Authors:
*         Michael Lehning <michael.lehning@lehning.de>
*
*/

#ifndef __SICK_SCAN_API_CONVERTER_H_INCLUDED
#define __SICK_SCAN_API_CONVERTER_H_INCLUDED

#if __ROS_VERSION == 1
#include <ros/ros.h>
#include <sensor_msgs/LaserScan.h>
#include <sensor_msgs/PointCloud2.h>
#include <sensor_msgs/Imu.h>
#include <visualization_msgs/MarkerArray.h>
#include <sick_scan_xd/LFErecMsg.h>             // LFErecMsg generated by ros msg-generator
#include <sick_scan_xd/LIDoutputstateMsg.h>     // LIDoutputstateMsg generated by ros msg-generator
#include <sick_scan_xd/RadarScan.h>             // RadarScan generated by ros msg-generator
#include <sick_scan_xd/SickLdmrsObjectArray.h>  // SickLdmrsObjectArray generated by ros msg-generator
#endif

#include "sick_scan_api.h"

class SickScanApiConverter
{
public:
#if __ROS_VERSION == 1
    /* Convert a cartesian SickScanPointCloudMsg to sensor_msgs::PointCloud2 (ROS-1 only) */
    static sensor_msgs::PointCloud2 convertPointCloudMsg(const SickScanPointCloudMsg& src_msg);

    /* Convert a polar SickScanPointCloudMsg to sensor_msgs::PointCloud2 (ROS-1 only) */
    static sensor_msgs::PointCloud2 convertPolarPointCloudMsg(const SickScanPointCloudMsg& src_msg);

    /* Convert a SickScanImuMsg to sensor_msgs::Imu (ROS-1 only) */
    static sensor_msgs::Imu convertImuMsg(const SickScanImuMsg& src_msg);

    /* Convert a SickScanLFErecMsg to sick_scan_xd::LFErecMsg (ROS-1 only) */
    static sick_scan_xd::LFErecMsg convertLFErecMsg(const SickScanLFErecMsg& src_msg);

    /* Convert a SickScanLIDoutputstateMsg to sick_scan_xd::LIDoutputstateMsg (ROS-1 only) */
    static sick_scan_xd::LIDoutputstateMsg convertLIDoutputstateMsg(const SickScanLIDoutputstateMsg& src_msg);

    /* Convert a SickScanLIDoutputstateMsg to sick_scan_xd::RadarScan (ROS-1 only) */
    static sick_scan_xd::RadarScan convertRadarScanMsg(const SickScanRadarScan& src_msg);

    /* Convert RadarScan objects to sensor_msgs::PointCloud2 (ROS-1 only) */
    static sensor_msgs::PointCloud2 convertRadarObjectsToPointCloud(const SickScanHeader& header, const sick_scan_xd::RadarObject* radar_objects, int num_objects);

    /* Convert a SickScanLdmrsObjectArray to sensor_msgs::SickLdmrsObjectArray (ROS-1 only) */
    static sick_scan_xd::SickLdmrsObjectArray convertLdmrsObjectArray(const SickScanLdmrsObjectArray& src_msg);

    /* Convert a SickScanVisualizationMarkerMsg to visualization_msgs::MarkerArray (ROS-1 only) */
    static visualization_msgs::MarkerArray convertVisualizationMarkerMsg(const SickScanVisualizationMarkerMsg& src_msg);

#endif

}; // SickScanApiConverter

#endif // __SICK_SCAN_API_CONVERTER_H_INCLUDED
